library stylight;

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter/material.dart';
import 'package:google_fonts/google_fonts.dart';

export 'website.dart';

part 'grid.dart';
part 'essentials.dart';

/*

Disclaimer:
Yes I know the TextTheme class, but i prefer having a "faster to implement" version for fast development

 */

class Sty {

  static final Sty sty = new Sty();

  // ignore: empty_constructor_bodies
  Sty() {

  }

  ThemeData get theme {
    return ThemeData(brightness: Brightness.dark, primarySwatch: primary, accentColor: Hues.Blue.hard, splashColor: Colors.transparent,
    scaffoldBackgroundColor: primary[500], cardColor: primary[800], buttonColor: Hues.Blue.hard,
      inputDecorationTheme: InputDecorationTheme(helperStyle: TextStyles.Footnote.style.copyWith(color: textColor3), labelStyle: TextStyles.Header.style.copyWith(color: textColor2)),
        tooltipTheme: TooltipThemeData(decoration: BoxDecoration(
          color: primary[800],
        ), textStyle: TextStyles.Body.style.copyWith(color: textColor1))
    );
  }

}

enum Hues {

  Blue,
  Red,
  Green,
  Yellow,
  Violet,
  Orange

}

//Using these font colors will probably replace the requirement of opacity in text
Color get textColor1 => Color(0xFFFFFFFF);
Color get textColor2 => Color(0xFFdcddde);
Color get textColor3 => Color(0xFF72767d);
Color get textColor4 => Color(0xFF5d5f63);

enum TextStyles {

  Title1,
  Title2,
  Title3,
  Header,
  SubHeader,
  Body,
  Footnote

}

TextTheme get textTheme{
  return TextTheme(
      bodyText1: headline, bodyText2: body, subtitle1: subHeadline,
      subtitle2: subHeadline, headline6: headline, headline5: title3,
      headline4: title2, headline3: title1, headline2: title1,
      headline1: title1
  );
}


const double _textBodySize = 16;

const double _textHeadlineSize = 16;
const double _textSubHeadlineSize = 14;

const double _textTitle3Size = 20;
const double _textTitle2Size = 22;
const double _textTitle1Size = 26;

const double _textCaptionFootnoteSize = 13;


TextStyle get title1 => TextStyle(fontSize: _textTitle1Size,  fontWeight: FontWeight.normal, color: textColor1);
TextStyle get title2 => TextStyle(fontSize: _textTitle2Size, fontWeight: FontWeight.normal, color: textColor1);
TextStyle get title3 => TextStyle(fontSize: _textTitle3Size, fontWeight: FontWeight.normal, color: textColor1);
TextStyle get headline => TextStyle(fontSize: _textHeadlineSize, fontWeight: FontWeight.bold, color: textColor3);
TextStyle get subHeadline => TextStyle(fontSize: _textSubHeadlineSize, fontWeight: FontWeight.normal, color: textColor2);
TextStyle get body => TextStyle(fontSize: _textBodySize,  fontWeight: FontWeight.normal, color: textColor2);
TextStyle get footnote => TextStyle(fontSize: _textCaptionFootnoteSize, fontWeight: FontWeight.normal, color: textColor4);


enum Fonts {
  Oswald,
  WorkSans,
  Ubuntu,
  Roboto,
  OpenSans
}

extension FontExt on Fonts {

  TextStyle apply(TextStyle style) {
    switch (this) {
      case Fonts.Oswald:
        return GoogleFonts.oswald(textStyle: style);
      case Fonts.WorkSans:
        return GoogleFonts.workSans(textStyle: style);
      case Fonts.Ubuntu:
        return GoogleFonts.ubuntu(textStyle: style);
      case Fonts.Roboto:
        return GoogleFonts.roboto(textStyle: style);
      case Fonts.OpenSans:
        return GoogleFonts.openSans(textStyle: style);
    }
  }

}

extension TextStylesExt on TextStyles {

  TextStyle _style() {
    switch (this) {

      case TextStyles.Title1:
        return title1;
      case TextStyles.Title2:
        return title2;
      case TextStyles.Title3:
        return title3;
      case TextStyles.Header:
        return headline;
      case TextStyles.SubHeader:
        return subHeadline;
      case TextStyles.Body:
        return body;
      case TextStyles.Footnote:
        return footnote;
    }
  }

  Widget text(String text, {TextStyle Function(TextStyle style) applied, Color color, FontWeight fontWeight, double fontSize, double padding = 0, Fonts font}) {
    TextStyle t = style.copyWith(color: color??style.color, fontWeight: fontWeight??style.fontWeight, fontSize: fontSize??style.fontSize);
    if (font != null) t = font.apply(t);
    if (applied != null) {
      t = applied(t);
    }
    Text textWidget = new Text(text, style: t);
    if (padding != 0) {
      return GridPadding(child: textWidget, multiple: padding,);
    } else {
      return textWidget;
    }
  }

  ///UseCase: Header
  Text oswald(String content) {
    return text(content, applied: (s) => GoogleFonts.oswald(textStyle: s));
  }

  //UseCase: Body
  Text workSans(String content) {
    return text(content, applied: (s) => GoogleFonts.workSans(textStyle: s));
  }

  //UseCase: Code
  Text ubuntu(String content) {
    return text(content, applied: (s) => GoogleFonts.ubuntu(textStyle: s));
  }

  //UseCase: Universal
  Text roboto(String content) {
    return text(content, applied: (s) => GoogleFonts.roboto(textStyle: s));
  }

  //UseCase: Universal
  Text openSans(String content) {
    return text(content, applied: (s) => GoogleFonts.openSans(textStyle: s));
  }

  TextStyle get style => _style();

}

//Primary color swatch in discord-like colors
const MaterialColor primary = MaterialColor(
    0xFF36393f,
    {
      100: Color(0xFFFFFFFF),
      200: Color(0xFFb9bbbe),
      300: Color(0xFF83878b),
      400: Color(0xFF3a3e44),

      500: Color(0xFF36393f),

      600: Color(0xFF2f3136),
      700: Color(0xFF292b2f),
      800: Color(0xFF202225),
      900: Color(0xFF000000),
    }
);

//Modern accent colors
extension HueExt on Hues {

  Color get tined => _tinted();
  Color get hard => _hard();

  Color _tinted() {
    switch (this) {
      case Hues.Blue:
        return Color(0xFF7289da);
      case Hues.Red:
        return Color(0xFFD67171);
      case Hues.Green:
        return Color(0xFF72D883);
      case Hues.Yellow:
        return Color(0xFFD1C170);
      case Hues.Violet:
        return Color(0xFF9F6FCE);
      case Hues.Orange:
        return Color(0xFFCC9D6E);
    }
  }

  Color _hard() {
    switch (this) {

      case Hues.Blue:
        return Color(0xFF7289da);
      case Hues.Red:
        return Color(0xFFf04747);
      case Hues.Green:
        return Color(0xFF43b581);
      case Hues.Yellow:
        return Color(0xFFffcc4d);
      case Hues.Violet:
        return Color(0xFFff73fa);
      case Hues.Orange:
        return Color(0xFFFAA61A);
    }
  }

}